function [gcarc,az,baz] = distaz(lat1,lon1,lat2,lon2)

% input: geographic coordinates in degrees
% output: geocentric distance, azimuth, and backazimuth in degrees

rad2deg = 180.0/pi;
deg2rad = pi/180.0;

geoc_lat2 = geocen(lat2);
geoc_lat2 = geoc_lat2*deg2rad;

clat2 = cos(geoc_lat2);
slat2 = sin(geoc_lat2);

% when the two locations are the same:
if lat1 == lat2 & lon1 == lon2
   gcarc = 0.0;
   az = 0.0;
   baz = 0.0;
   return
end

geoc_lat1 = geocen(lat1);
geoc_lat1 = geoc_lat1*deg2rad;

rdlon = deg2rad * (lon2 - lon1);

clat1 = cos(geoc_lat1);
slat1 = sin(geoc_lat1);


cdlon = cos(rdlon);
sdlon = sin(rdlon);

cdel = slat1 * slat2 + clat1 * clat2 * cdlon;

if (cdel < 1.0) 
    cdel = cdel;
elseif (cdel >= 1.0) 
    cdel = 1.;
end

if (cdel <= -1.0) 
    cdel = -1.;
end

yazi = sdlon * clat2;
xazi = clat1*slat2 - slat1*clat2*cdlon;
ybaz = -sdlon * clat1;
xbaz = clat2*slat1 - slat2*clat1*cdlon;

gcarc = rad2deg * acos(cdel);
az = rad2deg * atan2(yazi,xazi);
baz = rad2deg * atan2(ybaz,xbaz);

if (az < 0.0) 
   az = az + 360.0;
end
if (baz < 0.0) 
   baz = baz + 360.0;
end
returnfunction [glat] = geocen(alat)

rad2deg = 180.0/pi;
deg2rad = pi/180.0;

a = 6378.160;
ellip = 1.0/298.25;
b = a*(1.0-ellip);
gfact = (b*b)/(a*a);
glat = rad2deg * atan( gfact* (sin(alat*deg2rad)/cos(alat*deg2rad)) );
function [j] = locate(xx,N,x)

jl = 0;
ju = N+1;

while (ju-jl) > 1 
   jm = fix((ju+jl)/2);
   if ( xx(N,1) > xx(1,1) ) == ( x > xx(jm,1) )
       jl = jm;
   elseif (xx(N,1) > xx(1,1)) ~= (x > xx(jm,1) )
       ju = jm;
   end
end
j = jl;function [y,dy,d2y] = mysplin2(x1a,x2a,ya,y2a,M,N,x1,x2)

cutoff = 1.0*10^30;
for j = 1:M
  for k = 1:N
    ytmp(k,1) = ya(j,k);
    y2tmp(k,1) = y2a(j,k);
  end
  yytmp(j,1) = splint(x2a,ytmp,y2tmp,N,x2);
end

y2tmp = spline(x1a,yytmp,M,cutoff,cutoff);
[y,dy,d2y] = splint_deriv(x1a,yytmp,y2tmp,M,x1);function [x,z,tt] = read_tt_table(phase) 

% This function fills the 1D arrays x and z with the distances and
% depths at which the travel time curve is represented. The 2D array tt(nz,nx)
% will contain the P, or S, travel times at the sample points (zi,xi)

if phase == 'P' 
% a 59 x 5 table (295 values)
   d = load('distsamples.dat');
   szd = size(d);
   nx = szd(1)*szd(2);
   x = reshape(d,nx,1);  % rearrange distances into a column vector.
   x = sort(x); %sort so that the distances monotonically increase

% read in depth samples (in a row of 14 values)
   z = load('depthsamples.dat')';
   szd = size(z);
   nz = szd(1)*szd(2);

% read in P time, sampled at distance and depth samples
   p = load('P.dat');

% rearrange into a nz x nx matrix i = depth index, j = dist index
   for i = 1:nz
     for j = 1:nx
         k = j + (i-1)*nx;
         tt(i,j) = p(k);
     end
   end

elseif phase == 'S' 

% a 18 x 10 table (180 values)
   d = load('Sdist_samples');
   szd = size(d);
   nx = szd(1)*szd(2);
   x = reshape(d,nx,1);  % rearrange distances into a column vector.
   x = sort(x); % sort so that the distances monotonically increase

% read in depth samples (in a row of 14 values)
   z = load('depth_samples',-1,14)';
   szd = size(z);
   nz = szd(1)*szd(2);

% read in S time, sampled at distance and depth samples
   p = load('S.dat');

% rearrange into a nz x nx matrix i = depth index, j = dist index
   for i = 1:nz
     for j = 1:nx
         k = j + (i-1)*nx;
         tt(i,j) = p(k);
     end
    end

endfunction [y2a] = splie2(x1a,x2a,ya,M,N)

cutoff = 1.0*10^30;
for j = 1:M
  for k = 1:N
      ytmp(k,1) = ya(j,k);
  end
  y2tmp = spline(x2a,ytmp,N,cutoff,cutoff);
  for k = 1:N
     y2a(j,k) = y2tmp(k,1);
  end
endfunction [y2] = spline(x,y,N,yp1,ypn)

cutoff = 0.99*10^30;

if (yp1 > cutoff)
  y2(1,1) = 0.;
   u(1,1) = 0.;
elseif (yp1 <= cutoff)
  y2(1,1) = 0.5;
   u(1,1) = ( 3.0/(x(2,1) - x(1,1)) ) * ( (y(2,1)-y(1,1))/(x(2,1)-x(1,1)) - yp1);
end

for i=2:N-1
    sig =  ( x(i,1) - x(i-1,1) ) / ( x(i+1,1) - x(i,1)) ;
      p = sig * y2(i-1,1) + 2;
  y2(i,1) = (sig-1.0)/p;
   u(i,1) = (6.0 * ( (y(i+1,1)-y(i,1))/(x(i+1,1)-x(i,1)) - (y(i,1)-y(i-1,1)) / (x(i,1) - x(i-1,1)))/(x(i+1,1)-x(i-1,1))-sig*u(i-1,1))/p;
end

if (ypn > cutoff)
     qn = 0.0;
     un = 0.0;
elseif (ypn <= cutoff)
     qn = 0.5;
     un = (3.0 /(x(N,1)-x(N-1,1))) * (ypn- (y(N,1)-y(N-1,1))/(x(N,1)-x(N-1,1)));
end
y2(N,1) = (un - qn*u(N-1,1) )/(qn*y2(N-1,1)+1.0);
for i = 1:N-1
    k = N - i;
    y2(k,1) = y2(k,1)*y2(k+1,1) + u(k,1);
end
function [y] = splint(xa,ya,y2a,N,x)

klo = 1;
khi = N;

while ( (khi-klo) > 1 ) 
   k = (khi+klo)/2;
   k = fix(k);
   if ( xa(k,1) > x )
      khi = k;
   elseif (xa(k,1) <= x)
      klo = k;
   end
end

h = xa(khi,1) - xa(klo,1);

a = ( xa(khi,1) - x )/h;
b = ( x - xa(klo,1) )/h;
y = a*ya(klo,1) + b*ya(khi,1) + ((a*a*a - a)*y2a(klo,1) + (b*b*b - b)*y2a(khi,1))*(h*h)/6.0;
function [y,dy,d2y] = splint_deriv(xa,ya,y2a,N,x)

klo = 1;
khi = N;

while ( (khi-klo) > 1 ) 
   k = (khi+klo)/2;
   k = fix(k);
   if ( xa(k,1) > x )
      khi = k;
   elseif (xa(k,1) <= x)
      klo = k;
   end
end

h = xa(khi,1) - xa(klo,1);

a = ( xa(khi,1) - x )/h;
b = ( x - xa(klo,1) )/h;
y = a*ya(klo,1) + b*ya(khi,1) + ((a*a*a - a)*y2a(klo,1) + (b*b*b - b)*y2a(khi,1))*(h*h)/6.0;

dy = (ya(khi,1)-ya(klo,1))/h - ( (3.0*a*a - 1.0)*h*y2a(klo,1) )/6.0 + ( (3.0*b*b - 1.0)*h*y2a(khi,1) )/6.0;

d2y = a*y2a(klo,1) + b*y2a(khi,1);function [T,dTdh,dTdD] = ttime(x,h,dist,depths,tgrid)

% Routine to calculate travel times T, dTdh, and dTdD
% by bicubic spline interpolation of travel time curves that
% are read from pre-existing tables. Only P tables and S tables can be read.
% To obtain both dTdh and dTdD I followed the approach of W. Nagy (locsat).
%
% Input: x = epicentral distance in degrees
%        h = earthquake depth in km
%    phase = 'P' or 'S'
% Output:
%        T = the calculated travel time for (x,h) in s
%     dTdh = the estimate of dTdh in s/km
%          = derivative w.r.t. source depth = vertical slowness
%     dTdD = the estimate of dTdD  in s/deg
%          = derivative w.r.t. epicentral distance = ray parameter


% distance- and depth samples, and travel time values at those
% points are already read in by the main script.

szd = size(dist);
nx = szd(1)*szd(2);
szd = size(depths);
nz = szd(1)*szd(2);

% initialize output variables
T = zeros(1,1);
dTdD = zeros(1,1);
dTdh = zeros(1,1);

% determine the subset of the travel time curve to interpolate with splines.

nxmax = 7;
nzmax = 7;

% [jleft] = call('locate',dist,1,'r',nx,2,'i',x,3,'r','out',[1,1],4,'i');
% [ileft] = call('locate',depths,1,'r',nz,2,'i',h,3,'r','out',[1,1],4,'i');

[jleft] = locate(dist,nx,x);
[ileft] = locate(depths,nz,h);

imin = ileft - 3;
imax = ileft + 3;
if (imin) <= 0 
   imin = 1;
   imax = nzmax;
end
if (imax) > nz 
   imax = nz;
   imin = nz - nzmax;
end
jmin = jleft - 3;
jmax = jleft + 3;
if (jmin) <= 0 
   jmin = 1;
   jmax = nxmax;
end
if (jmax) > nx 
   jmax = nx;
   jmin = nx - nxmax;
end
d = dist(jmin:jmax);
t = tgrid(imin:imax,jmin:jmax);
z = depths(imin:imax);
nd = (jmax - jmin) + 1;
nh = (imax - imin) + 1;
dummy = 2e30;

% First interpolate the rows of the t-matrix and create a vector with 2nd
% derivatives:

% [y2atrans] = call('splie2',d,1,'r',z,2,'r',t',3,'r',nd,4,'i',nh,5,'i','out',[nd,nh],6,'r');
[y2atrans] = splie2(d,z,t',nd,nh);

% run mysplin2 to find T and dTdD
%[value,dTdD,dT2dD] = call('mysplin2',d,1,'r',z,2,'r',t',3,'r',y2atrans,4,'r',nn,5,'i',nn,6,'i',x,7,'r',h,8,'r','out',[1,1],9,'r',[1,1],10,'r',[1,1],11,'r') 

[value,dTdD,dT2dD] = mysplin2(d,z,t',y2atrans,nd,nh,x,h);

% Repeat with the transpose
%[y2a] = call('splie2',z,1,'r',d,2,'r',t,3,'r',nn,4,'i',nn,5,'i','out',[nn,nn],6,'r');
[y2a] = splie2(z,d,t,nh,nd);

% Then calculate the value of T and dTdh (and dT2dh)
%[value,dTdh,dT2dh] = call('mysplin2',z,1,'r',d,2,'r',t,3,'r',y2a,4,'r',nn,5,'i',nn,6,'i',h,7,'r',x,8,'r','out',[1,1],9,'r',[1,1],10,'r',[1,1],11,'r')

[value,dTdh,dT2dh] = mysplin2(z,d,t,y2a,nh,nd,h,x);

T = value;