function [gcarc,az,baz] = distaz(lat1,lon1,lat2,lon2)

% input: geographic coordinates in degrees
% output: geocentric distance, azimuth, and backazimuth in degrees

rad2deg = 180.0/pi;
deg2rad = pi/180.0;

geoc_lat2 = geocen(lat2);
geoc_lat2 = geoc_lat2*deg2rad;

clat2 = cos(geoc_lat2);
slat2 = sin(geoc_lat2);

% when the two locations are the same:
if lat1 == lat2 & lon1 == lon2
   gcarc = 0.0;
   az = 0.0;
   baz = 0.0;
   return
end

geoc_lat1 = geocen(lat1);
geoc_lat1 = geoc_lat1*deg2rad;

rdlon = deg2rad * (lon2 - lon1);

clat1 = cos(geoc_lat1);
slat1 = sin(geoc_lat1);


cdlon = cos(rdlon);
sdlon = sin(rdlon);

cdel = slat1 * slat2 + clat1 * clat2 * cdlon;

if (cdel < 1.0) 
    cdel = cdel;
elseif (cdel >= 1.0) 
    cdel = 1.;
end

if (cdel <= -1.0) 
    cdel = -1.;
end

yazi = sdlon * clat2;
xazi = clat1*slat2 - slat1*clat2*cdlon;
ybaz = -sdlon * clat1;
xbaz = clat2*slat1 - slat2*clat1*cdlon;

gcarc = rad2deg * acos(cdel);
az = rad2deg * atan2(yazi,xazi);
baz = rad2deg * atan2(ybaz,xbaz);

if (az < 0.0) 
   az = az + 360.0;
end
if (baz < 0.0) 
   baz = baz + 360.0;
end
return