function [x,z,tt] = read_tt_table(phase) 

% This function fills the 1D arrays x and z with the distances and
% depths at which the travel time curve is represented. The 2D array tt(nz,nx)
% will contain the P, or S, travel times at the sample points (zi,xi)

if phase == 'P' 
% a 59 x 5 table (295 values)
   d = load('distsamples.dat');
   szd = size(d);
   nx = szd(1)*szd(2);
   x = reshape(d,nx,1);  % rearrange distances into a column vector.
   x = sort(x); %sort so that the distances monotonically increase

% read in depth samples (in a row of 14 values)
   z = load('depthsamples.dat')';
   szd = size(z);
   nz = szd(1)*szd(2);

% read in P time, sampled at distance and depth samples
   p = load('P.dat');

% rearrange into a nz x nx matrix i = depth index, j = dist index
   for i = 1:nz
     for j = 1:nx
         k = j + (i-1)*nx;
         tt(i,j) = p(k);
     end
   end

elseif phase == 'S' 

% a 18 x 10 table (180 values)
   d = load('Sdist_samples');
   szd = size(d);
   nx = szd(1)*szd(2);
   x = reshape(d,nx,1);  % rearrange distances into a column vector.
   x = sort(x); % sort so that the distances monotonically increase

% read in depth samples (in a row of 14 values)
   z = load('depth_samples',-1,14)';
   szd = size(z);
   nz = szd(1)*szd(2);

% read in S time, sampled at distance and depth samples
   p = load('S.dat');

% rearrange into a nz x nx matrix i = depth index, j = dist index
   for i = 1:nz
     for j = 1:nx
         k = j + (i-1)*nx;
         tt(i,j) = p(k);
     end
    end

end